//############################################################################
//### Name:        ShowerTrackStartPosition                                ###
//### Author:      Dom Barker                                              ###
//### Date:        13.05.19                                                ###
//### Description: Tool for settung the shower start position to the       ###
//                 start of the fitted track                               ###
//############################################################################

//Framework Includes
#include "art/Utilities/ToolMacros.h"

//LArSoft Includes
#include "lardataobj/RecoBase/Track.h"
#include "larpandora/LArPandoraEventBuilding/LArPandoraShower/Tools/IShowerTool.h"

namespace ShowerRecoTools {

  class ShowerTrackStartPosition : public IShowerTool {

  public:
    ShowerTrackStartPosition(const fhicl::ParameterSet& pset);

    //Generic Direction Finder
    int CalculateElement(const art::Ptr<recob::PFParticle>& pfparticle,
                         art::Event& Event,
                         reco::shower::ShowerElementHolder& ShowerEleHolder) override;

  private:
    int fVerbose;
    std::string fInitialTrackInputLabel;
    std::string fShowerStartPositionOutputLabel;
  };

  ShowerTrackStartPosition::ShowerTrackStartPosition(const fhicl::ParameterSet& pset)
    : IShowerTool(pset.get<fhicl::ParameterSet>("BaseTools"))
    , fVerbose(pset.get<int>("Verbose"))
    , fInitialTrackInputLabel(pset.get<std::string>("InitialTrackInputLabel"))
    , fShowerStartPositionOutputLabel(pset.get<std::string>("ShowerStartPositionOutputLabel"))
  {}

  int ShowerTrackStartPosition::CalculateElement(const art::Ptr<recob::PFParticle>& pfparticle,
                                                 art::Event& Event,
                                                 reco::shower::ShowerElementHolder& ShowerEleHolder)
  {

    //Check the Track has been defined
    if (!ShowerEleHolder.CheckElement("InitialTrack")) {
      if (fVerbose)
        mf::LogError("ShowerTrackStartPosition") << "Initial track not set" << std::endl;
      return 1;
    }
    recob::Track InitialTrack;
    ShowerEleHolder.GetElement(fInitialTrackInputLabel, InitialTrack);

    //Set the shower start position as the
    geo::Point_t StartPositionErr = {-999, -999, -999};

    geo::Point_t TrajPosition = InitialTrack.LocationAtPoint(0);
    ShowerEleHolder.SetElement(TrajPosition, StartPositionErr, fShowerStartPositionOutputLabel);

    return 0;
  }
}

DEFINE_ART_CLASS_TOOL(ShowerRecoTools::ShowerTrackStartPosition)
